@@ -72,7 +72,7 @@
 //         fmt.Printf("%d errors and %d warnings\n",
 //             len(result.Errors), len(result.Warnings))
 //
-//         os.Stdout.Write(result.JS)
+//         os.Stdout.Write(result.Code)
 //     }
 //
 package api
@@ -102,7 +102,8 @@ const (
 type Loader uint8
 
 const (
-	LoaderJS Loader = iota
+	LoaderNone Loader = iota
+	LoaderJS
 	LoaderJSX
 	LoaderTS
 	LoaderTSX
@@ -113,6 +114,7 @@ const (
 	LoaderFile
 	LoaderBinary
 	LoaderCSS
+	LoaderDefault
 )
 
 type Platform uint8
@@ -148,11 +150,12 @@ type Engine struct {
 }
 
 type Location struct {
-	File     string
-	Line     int // 1-based
-	Column   int // 0-based, in bytes
-	Length   int // in bytes
-	LineText string
+	File      string
+	Namespace string
+	Line      int // 1-based
+	Column    int // 0-based, in bytes
+	Length    int // in bytes
+	LineText  string
 }
 
 type Message struct {
@@ -177,11 +180,13 @@ const (
 	LogLevelError
 )
 
-type StrictOptions struct {
-	NullishCoalescing bool
-	OptionalChaining  bool
-	ClassFields       bool
-}
+type Charset uint8
+
+const (
+	CharsetDefault Charset = iota
+	CharsetASCII
+	CharsetUTF8
+)
 
 ////////////////////////////////////////////////////////////////////////////////
 // Build API
@@ -194,18 +199,18 @@ type BuildOptions struct {
 	Sourcemap SourceMap
 	Target    Target
 	Engines   []Engine
-	Strict    StrictOptions
 
 	MinifyWhitespace  bool
 	MinifyIdentifiers bool
 	MinifySyntax      bool
+	Charset           Charset
 
 	JSXFactory  string
 	JSXFragment string
 
-	Defines       map[string]string
-	PureFunctions []string
-	AvoidTDZ      bool
+	Define   map[string]string
+	Pure     []string
+	AvoidTDZ bool
 
 	GlobalName        string
 	Bundle            bool
@@ -213,11 +218,12 @@ type BuildOptions struct {
 	Outfile           string
 	Metafile          string
 	Outdir            string
+	Outbase           string
 	Platform          Platform
 	Format            Format
-	Externals         []string
+	External          []string
 	MainFields        []string
-	Loaders           map[string]Loader
+	Loader            map[string]Loader
 	ResolveExtensions []string
 	Tsconfig          string
 	OutExtensions     map[string]string
@@ -227,6 +233,7 @@ type BuildOptions struct {
 	EntryPoints []string
 	Stdin       *StdinOptions
 	Write       bool
+	Plugins     []Plugin
 }
 
 type StdinOptions struct {
@@ -265,18 +272,19 @@ type TransformOptions struct {
 	Format     Format
 	GlobalName string
 	Engines    []Engine
-	Strict     StrictOptions
 
 	MinifyWhitespace  bool
 	MinifyIdentifiers bool
 	MinifySyntax      bool
+	Charset           Charset
 
 	JSXFactory  string
 	JSXFragment string
+	TsconfigRaw string
 
-	Defines       map[string]string
-	PureFunctions []string
-	AvoidTDZ      bool
+	Define   map[string]string
+	Pure     []string
+	AvoidTDZ bool
 
 	Sourcefile string
 	Loader     Loader
@@ -286,10 +294,67 @@ type TransformResult struct {
 	Errors   []Message
 	Warnings []Message
 
-	JS          []byte
-	JSSourceMap []byte
+	Code []byte
+	Map  []byte
 }
 
 func Transform(input string, options TransformOptions) TransformResult {
 	return transformImpl(input, options)
 }
+
+////////////////////////////////////////////////////////////////////////////////
+// Plugin API
+
+type Plugin struct {
+	Name  string
+	Setup func(PluginBuild)
+}
+
+type PluginBuild interface {
+	OnResolve(options OnResolveOptions, callback func(OnResolveArgs) (OnResolveResult, error))
+	OnLoad(options OnLoadOptions, callback func(OnLoadArgs) (OnLoadResult, error))
+}
+
+type OnResolveOptions struct {
+	Filter    string
+	Namespace string
+}
+
+type OnResolveArgs struct {
+	Path       string
+	Importer   string
+	Namespace  string
+	ResolveDir string
+}
+
+type OnResolveResult struct {
+	PluginName string
+
+	Errors   []Message
+	Warnings []Message
+
+	Path      string
+	External  bool
+	Namespace string
+}
+
+type OnLoadOptions struct {
+	Filter    string
+	Namespace string
+}
+
+type OnLoadArgs struct {
+	Path      string
+	Namespace string
+}
+
+type OnLoadResult struct {
+	PluginName string
+
+	Errors   []Message
+	Warnings []Message
+
+	Contents   *string
+	ResolveDir string
+	Loader     Loader
+}
